==> joynr-1.21.4.tgz: package/joynr/capabilities/arbitration/Arbitrator.js <==
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
const DiscoveryError_1 = __importDefault(require("../../../generated/joynr/types/DiscoveryError"));
const LoggingManager_1 = __importDefault(require("../../system/LoggingManager"));
const UtilInternal = __importStar(require("../../util/UtilInternal"));
const DiscoveryException_1 = __importDefault(require("../../exceptions/DiscoveryException"));
const NoCompatibleProviderFoundException_1 = __importDefault(require("../../exceptions/NoCompatibleProviderFoundException"));
const LongTimer_1 = __importDefault(require("../../util/LongTimer"));
const DiscoveryQosGen = require("../../../generated/joynr/types/DiscoveryQos");
const ApplicationException = require("../../exceptions/ApplicationException");
const ArbitrationStrategyCollection = __importStar(require("../../../joynr/types/ArbitrationStrategyCollection"));
const ArbitrationConstants_1 = require("../../types/ArbitrationConstants");
const log = LoggingManager_1.default.getLogger("joynr.capabilities.arbitration.Arbitrator");
function addToListIfNotExisting(list, providerVersion) {
    for (let j = 0; j < list.length; ++j) {
        if (list[j].majorVersion === providerVersion.majorVersion &&
            list[j].minorVersion === providerVersion.minorVersion) {
            return;
        }
    }
    list.push(providerVersion);
}
class Arbitrator {
    /**
     * An arbitrator looks up all capabilities for given domains and an interface and uses the provides arbitraionStrategy passed in the
     * discoveryQos to choose one or more for the calling proxy
     *
     * @constructor
     *
     * @param capabilityDiscoveryStub the capability discovery
     * @param staticCapabilities the capabilities the arbitrator will use to resolve capabilities in case of static arbitration
     */
    constructor(capabilityDiscoveryStub, staticCapabilities) {
        this.started = true;
        this.arbitrationId = 0;
        this.pendingArbitrations = {};
        this.staticCapabilities = staticCapabilities;
        this.capabilityDiscoveryStub = capabilityDiscoveryStub;
    }
    /**
     * Starts the arbitration process
     *
     * @param settings the settings object
     * @param settings.domains the domains to discover the provider
     * @param settings.interfaceName the interfaceName to discover the provider
     * @param settings.discoveryQos
     * @param [settings.staticArbitration] shall the arbitrator use staticCapabilities or contact the discovery provider
     * @param [settings.proxyVersion] the version of the proxy object
     *
     * @returns Promise
     *  - resolved with an array of arbitrated capabilities
     *  - rejected with either DiscoveryException or NoCompatibleProviderFoundException
     */
    async startArbitration(settings) {
        if (!this.started) {
            return Promise.reject(new Error("Arbitrator is already shut down"));
        }
        this.arbitrationId++;
        log.debug(`Arbitration started for domains ${settings.domains}, interface ${settings.interfaceName}, gbids ${settings.gbids}.`);
        if (settings.staticArbitration && this.staticCapabilities) {
            return this.discoverStaticCapabilities(settings.domains, settings.interfaceName, settings.discoveryQos, settings.proxyVersion);
        }
        else {
            return this.discoverCapabilitiesWrapper(settings);
        }
    }
    /**
     * checks if the provided discoveryEntry supports onChange subscriptions if required
     *
     * @param discoveryEntry - the discovery entry to check
     * @param providerMustSupportOnChange - filter only entries supporting onChange subscriptions
     */
    static checkSupportsOnChangeSubscriptions(discoveryEntry, providerMustSupportOnChange) {
        if (providerMustSupportOnChange === undefined || !providerMustSupportOnChange) {
            return true;
        }
        if (discoveryEntry.qos === undefined) {
            return false;
        }
        return discoveryEntry.qos.supportsOnChangeSubscriptions;
    }
    discoverCapabilitiesWrapper(settings) {
        const id = this.arbitrationId;
        const deferred = UtilInternal.createDeferred();
        this.pendingArbitrations[id] = deferred;
        this.discoverCapabilities({
            capabilityDiscoveryStub: this.capabilityDiscoveryStub,
            domains: settings.domains,
            interfaceName: settings.interfaceName,
            discoveryQos: settings.discoveryQos,
            proxyVersion: settings.proxyVersion,
            gbids: settings.gbids || []
        })
            .then(args => {
            delete this.pendingArbitrations[id];
            deferred.resolve(args);
        })
            .catch(error => {
            delete this.pendingArbitrations[id];
            deferred.reject(error);
        });
        return deferred.promise;
    }
    /**
     * Tries to discover capabilities with given domains, interfaceName and discoveryQos within the localCapDir as long as the deferred's state is pending
     *
     * @param capabilityDiscoveryStub - the capabilites discovery module
     * @param domains - the domains
     * @param interfaceName - the interfaceName
     * @param applicationDiscoveryQos
     * @param proxyVersion
     * @param gbids global backend identifiers of backends to look for capabilities
     * @returns a Promise/A+ object, that will provide an array of discovered capabilities
     */
    async discoverCapabilities({ capabilityDiscoveryStub, domains, interfaceName, discoveryQos, proxyVersion, gbids }) {
        // discover caps from local capabilities directory
        let incompatibleVersionsFound = [];
        const arbitrationDeadline = Date.now() + discoveryQos.discoveryTimeoutMs;
        const discoveryRetryDelayMs = discoveryQos.discoveryRetryDelayMs;
        let errorMsg = null;
        let firstLoop = true;
        let participantId;
        const isArbitrationStrategyFixedParticipant = discoveryQos.arbitrationStrategy === ArbitrationStrategyCollection.FixedParticipant;
        participantId = "";
        if (isArbitrationStrategyFixedParticipant) {
            if (!Object.prototype.hasOwnProperty.call(discoveryQos.additionalParameters, ArbitrationConstants_1.FIXED_PARTICIPANT_PARAMETER)) {
                throw new Error("parameter FIXED_PARTICIPANT_PARAMETER does not exist in DiscoveryQos.additionalParameters");
            }
            participantId = discoveryQos.additionalParameters[ArbitrationConstants_1.FIXED_PARTICIPANT_PARAMETER];
        }
        do {
            if (!firstLoop) {
                // eslint-disable-next-line promise/avoid-new
                await new Promise(resolve => {
                    LongTimer_1.default.setTimeout(resolve, discoveryRetryDelayMs);
                });
            }
            firstLoop = false;
            incompatibleVersionsFound = [];
            let discoveredCaps;
            try {
                if (isArbitrationStrategyFixedParticipant) {
                    discoveredCaps = [];
                    discoveredCaps.push(await capabilityDiscoveryStub.lookupByParticipantId(participantId, new DiscoveryQosGen({
                        discoveryScope: discoveryQos.discoveryScope,
                        cacheMaxAge: discoveryQos.cacheMaxAgeMs,
                        discoveryTimeout: arbitrationDeadline - Date.now(),
                        providerMustSupportOnChange: discoveryQos.providerMustSupportOnChange
                    }), gbids));
                    if (discoveredCaps.length > 0) {
                        if (discoveredCaps[0].interfaceName !== interfaceName) {
                            const errorMsg = `Interface "${discoveredCaps[0].interfaceName}" of discovered provider does not match proxy's interface "${interfaceName}".`;
                            log.error(errorMsg);
                            return Promise.reject(errorMsg);
                        }
                    }
                }
                else {
                    discoveredCaps = await capabilityDiscoveryStub.lookup(domains, interfaceName, new DiscoveryQosGen({
                        discoveryScope: discoveryQos.discoveryScope,
                        cacheMaxAge: discoveryQos.cacheMaxAgeMs,
                        discoveryTimeout: arbitrationDeadline - Date.now(),
                        providerMustSupportOnChange: discoveryQos.providerMustSupportOnChange
                    }), gbids);
                }
                const versionCompatibleCaps = [];
                for (let i = 0; i < discoveredCaps.length; i++) {
                    const providerVersion = discoveredCaps[i].providerVersion;
                    if (Arbitrator.checkSupportsOnChangeSubscriptions(discoveredCaps[i], discoveryQos.providerMustSupportOnChange)) {
                        if (providerVersion.majorVersion === proxyVersion.majorVersion &&
                            providerVersion.minorVersion >= proxyVersion.minorVersion) {
                            versionCompatibleCaps.push(discoveredCaps[i]);
                        }
                        else {
                            addToListIfNotExisting(incompatibleVersionsFound, providerVersion);
                        }
                    }
                }
                // filter caps according to chosen arbitration strategy
                const arbitratedCaps = discoveryQos.arbitrationStrategy(versionCompatibleCaps);
                if (arbitratedCaps.length > 0) {
                    // report the discovered & arbitrated caps
                    return arbitratedCaps;
                }
            }
            catch (error) {
                if (error instanceof ApplicationException) {
                    errorMsg = `Discovery failed due to ${error.error.name}`;
                    if (error.error !== DiscoveryError_1.default.NO_ENTRY_FOR_PARTICIPANT &&
                        error.error !== DiscoveryError_1.default.NO_ENTRY_FOR_SELECTED_BACKENDS) {
                        log.error(`Discovery attempt for domains ${domains}, interface ${interfaceName}, gbids ${gbids} failed due to DiscoveryError: ${error.error.name}. Attempting no retry`);
                        break;
                    }
                    else {
                        log.info(`Discovery attempt for domains ${domains}, interface ${interfaceName}, gbids ${gbids} failed due to DiscoveryError: ${error.error.name}. Attempting retry in ${discoveryQos.discoveryRetryDelayMs} ms`);
                    }
                }
                else if (error.message) {
                    log.info(`Discovery attempt for domains ${domains}, interface ${interfaceName} failed due to DiscoveryError: ${error.name}. Attempting retry in ${discoveryQos.discoveryRetryDelayMs} ms`);
                    errorMsg = `${error.name} : ${error.message}`;
                }
            }
        } while (arbitrationDeadline - (Date.now() + discoveryRetryDelayMs) > 0);
        if (incompatibleVersionsFound.length > 0) {
            let message;
            if (gbids && gbids.length > 0) {
                message = `no compatible provider found within discovery timeout for domains "${JSON.stringify(domains)}", interface "${interfaceName}", gbids "${JSON.stringify(gbids)}" with discoveryQos "${JSON.stringify(discoveryQos)}"`;
            }
            else {
                message = `no compatible provider found within discovery timeout for domains "${JSON.stringify(domains)}", interface "${interfaceName}" with discoveryQos "${JSON.stringify(discoveryQos)}"`;
            }
            return Promise.reject(new NoCompatibleProviderFoundException_1.default({
                detailMessage: message,
                discoveredVersions: incompatibleVersionsFound,
                interfaceName
            }));
        }
        else {
            let message;
            if (gbids && gbids.length > 0) {
                message = `no provider found within discovery timeout for domains "${JSON.stringify(domains)}", interface "${interfaceName}", gbids "${JSON.stringify(gbids)}" with discoveryQos "${JSON.stringify(discoveryQos)}"${errorMsg !== undefined ? `. Error: ${errorMsg}` : ""}`;
            }
            else {
                message = `no provider found within discovery timeout for domains "${JSON.stringify(domains)}", interface "${interfaceName}" with discoveryQos "${JSON.stringify(discoveryQos)}"${errorMsg !== undefined ? `. Error: ${errorMsg}` : ""}`;
            }
            return Promise.reject(new DiscoveryException_1.default({
                detailMessage: message
            }));
        }
    }
    async discoverStaticCapabilities(domains, interfaceName, discoveryQos, proxyVersion) {
        try {
            const arbitratedCaps = [];
            if (this.staticCapabilities === undefined) {
                return Promise.reject(new Error("Exception while arbitrating: static capabilities are missing"));
            }
            else {
                for (let i = 0; i < this.staticCapabilities.length; ++i) {
                    const capability = this.staticCapabilities[i];
                    if (domains.indexOf(capability.domain) !== -1 &&
                        interfaceName === capability.interfaceName &&
                        capability.providerVersion.majorVersion === proxyVersion.majorVersion &&
                        capability.providerVersion.minorVersion >= proxyVersion.minorVersion &&
                        Arbitrator.checkSupportsOnChangeSubscriptions(capability, discoveryQos.providerMustSupportOnChange)) {
                        arbitratedCaps.push(capability);
                    }
                }
                return discoveryQos.arbitrationStrategy(arbitratedCaps);
            }
        }
        catch (e) {
            return Promise.reject(new Error(`Exception while arbitrating: ${e}`));
        }
    }
    /**
     * Shutdown the Arbitrator
     */
    shutdown() {
        for (const id in this.pendingArbitrations) {
            if (Object.prototype.hasOwnProperty.call(this.pendingArbitrations, id)) {
                const pendingArbitration = this.pendingArbitrations[id];
                pendingArbitration.reject(new Error("Arbitration is already shut down"));
            }
        }
        this.pendingArbitrations = {};
        this.started = false;
    }
}
module.exports = Arbitrator;

==> joynr-1.21.7.tgz: package/joynr/capabilities/arbitration/Arbitrator.js <==
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
const DiscoveryError_1 = __importDefault(require("../../../generated/joynr/types/DiscoveryError"));
const LoggingManager_1 = __importDefault(require("../../system/LoggingManager"));
const UtilInternal = __importStar(require("../../util/UtilInternal"));
const DiscoveryException_1 = __importDefault(require("../../exceptions/DiscoveryException"));
const NoCompatibleProviderFoundException_1 = __importDefault(require("../../exceptions/NoCompatibleProviderFoundException"));
const LongTimer_1 = __importDefault(require("../../util/LongTimer"));
const DiscoveryQosGen = require("../../../generated/joynr/types/DiscoveryQos");
const ApplicationException = require("../../exceptions/ApplicationException");
const ArbitrationStrategyCollection = __importStar(require("../../../joynr/types/ArbitrationStrategyCollection"));
const ArbitrationConstants_1 = require("../../types/ArbitrationConstants");
const log = LoggingManager_1.default.getLogger("joynr.capabilities.arbitration.Arbitrator");
function addToListIfNotExisting(list, providerVersion) {
    for (let j = 0; j < list.length; ++j) {
        if (list[j].majorVersion === providerVersion.majorVersion &&
            list[j].minorVersion === providerVersion.minorVersion) {
            return;
        }
    }
    list.push(providerVersion);
}
class Arbitrator {
    /**
     * An arbitrator looks up all capabilities for given domains and an interface and uses the provides arbitraionStrategy passed in the
     * discoveryQos to choose one or more for the calling proxy
     *
     * @constructor
     *
     * @param capabilityDiscoveryStub the capability discovery
     * @param staticCapabilities the capabilities the arbitrator will use to resolve capabilities in case of static arbitration
     */
    constructor(capabilityDiscoveryStub, staticCapabilities) {
        this.started = true;
        this.arbitrationId = 0;
        this.pendingArbitrations = {};
        this.staticCapabilities = staticCapabilities;
        this.capabilityDiscoveryStub = capabilityDiscoveryStub;
    }
    /**
     * Starts the arbitration process
     *
     * @param settings the settings object
     * @param settings.domains the domains to discover the provider
     * @param settings.interfaceName the interfaceName to discover the provider
     * @param settings.discoveryQos
     * @param [settings.staticArbitration] shall the arbitrator use staticCapabilities or contact the discovery provider
     * @param [settings.proxyVersion] the version of the proxy object
     *
     * @returns Promise
     *  - resolved with an array of arbitrated capabilities
     *  - rejected with either DiscoveryException or NoCompatibleProviderFoundException
     */
    async startArbitration(settings) {
        if (!this.started) {
            return Promise.reject(new Error("Arbitrator is already shut down"));
        }
        this.arbitrationId++;
        log.debug(`Arbitration started for domains ${settings.domains}, interface ${settings.interfaceName}, gbids ${settings.gbids}.`);
        if (settings.staticArbitration && this.staticCapabilities) {
            return this.discoverStaticCapabilities(settings.domains, settings.interfaceName, settings.discoveryQos, settings.proxyVersion);
        }
        else {
            return this.discoverCapabilitiesWrapper(settings);
        }
    }
    /**
     * checks if the provided discoveryEntry supports onChange subscriptions if required
     *
     * @param discoveryEntry - the discovery entry to check
     * @param providerMustSupportOnChange - filter only entries supporting onChange subscriptions
     */
    static checkSupportsOnChangeSubscriptions(discoveryEntry, providerMustSupportOnChange) {
        if (providerMustSupportOnChange === undefined || !providerMustSupportOnChange) {
            return true;
        }
        if (discoveryEntry.qos === undefined) {
            return false;
        }
        return discoveryEntry.qos.supportsOnChangeSubscriptions;
    }
    discoverCapabilitiesWrapper(settings) {
        const id = this.arbitrationId;
        const deferred = UtilInternal.createDeferred();
        this.pendingArbitrations[id] = deferred;
        this.discoverCapabilities({
            capabilityDiscoveryStub: this.capabilityDiscoveryStub,
            domains: settings.domains,
            interfaceName: settings.interfaceName,
            discoveryQos: settings.discoveryQos,
            proxyVersion: settings.proxyVersion,
            gbids: settings.gbids || []
        })
            .then(args => {
            delete this.pendingArbitrations[id];
            deferred.resolve(args);
        })
            .catch(error => {
            delete this.pendingArbitrations[id];
            deferred.reject(error);
        });
        return deferred.promise;
    }
    /**
     * Tries to discover capabilities with given domains, interfaceName and discoveryQos within the localCapDir as long as the deferred's state is pending
     *
     * @param capabilityDiscoveryStub - the capabilites discovery module
     * @param domains - the domains
     * @param interfaceName - the interfaceName
     * @param applicationDiscoveryQos
     * @param proxyVersion
     * @param gbids global backend identifiers of backends to look for capabilities
     * @returns a Promise/A+ object, that will provide an array of discovered capabilities
     */
    async discoverCapabilities({ capabilityDiscoveryStub, domains, interfaceName, discoveryQos, proxyVersion, gbids }) {
        // discover caps from local capabilities directory
        let incompatibleVersionsFound = [];
        const arbitrationDeadline = Date.now() + discoveryQos.discoveryTimeoutMs;
        const discoveryRetryDelayMs = discoveryQos.discoveryRetryDelayMs;
        let errorMsg = null;
        let firstLoop = true;
        let participantId;
        const isArbitrationStrategyFixedParticipant = discoveryQos.arbitrationStrategy === ArbitrationStrategyCollection.FixedParticipant;
        participantId = "";
        if (isArbitrationStrategyFixedParticipant) {
            if (!Object.prototype.hasOwnProperty.call(discoveryQos.additionalParameters, ArbitrationConstants_1.FIXED_PARTICIPANT_PARAMETER)) {
                throw new Error("parameter FIXED_PARTICIPANT_PARAMETER does not exist in DiscoveryQos.additionalParameters");
            }
            participantId = discoveryQos.additionalParameters[ArbitrationConstants_1.FIXED_PARTICIPANT_PARAMETER];
        }
        do {
            if (!firstLoop) {
                // eslint-disable-next-line promise/avoid-new
                await new Promise(resolve => {
                    LongTimer_1.default.setTimeout(resolve, discoveryRetryDelayMs);
                });
            }
            firstLoop = false;
            incompatibleVersionsFound = [];
            let discoveredCaps;
            try {
                if (isArbitrationStrategyFixedParticipant) {
                    discoveredCaps = [];
                    discoveredCaps.push(await capabilityDiscoveryStub.lookupByParticipantId(participantId, new DiscoveryQosGen({
                        discoveryScope: discoveryQos.discoveryScope,
                        cacheMaxAge: discoveryQos.cacheMaxAgeMs,
                        discoveryTimeout: arbitrationDeadline - Date.now(),
                        providerMustSupportOnChange: discoveryQos.providerMustSupportOnChange
                    }), gbids));
                    if (discoveredCaps.length > 0) {
                        if (discoveredCaps[0].interfaceName !== interfaceName) {
                            const errorMsg = `Interface "${discoveredCaps[0].interfaceName}" of discovered provider does not match proxy's interface "${interfaceName}".`;
                            log.error(errorMsg);
                            return Promise.reject(errorMsg);
                        }
                    }
                }
                else {
                    discoveredCaps = await capabilityDiscoveryStub.lookup(domains, interfaceName, new DiscoveryQosGen({
                        discoveryScope: discoveryQos.discoveryScope,
                        cacheMaxAge: discoveryQos.cacheMaxAgeMs,
                        discoveryTimeout: arbitrationDeadline - Date.now(),
                        providerMustSupportOnChange: discoveryQos.providerMustSupportOnChange
                    }), gbids);
                }
                const versionCompatibleCaps = [];
                for (let i = 0; i < discoveredCaps.length; i++) {
                    const providerVersion = discoveredCaps[i].providerVersion;
                    if (Arbitrator.checkSupportsOnChangeSubscriptions(discoveredCaps[i], discoveryQos.providerMustSupportOnChange)) {
                        if (providerVersion.majorVersion === proxyVersion.majorVersion &&
                            providerVersion.minorVersion >= proxyVersion.minorVersion) {
                            versionCompatibleCaps.push(discoveredCaps[i]);
                        }
                        else {
                            addToListIfNotExisting(incompatibleVersionsFound, providerVersion);
                        }
                    }
                }
                // filter caps according to chosen arbitration strategy
                const arbitratedCaps = discoveryQos.arbitrationStrategy(versionCompatibleCaps);
                if (arbitratedCaps.length > 0) {
                    // report the discovered & arbitrated caps
                    return arbitratedCaps;
                }
            }
            catch (error) {
                if (error instanceof ApplicationException) {
                    errorMsg = `Discovery failed due to DiscoveryError ${error.error.name} : ${error.detailMessage}`;
                    if (error.error !== DiscoveryError_1.default.NO_ENTRY_FOR_PARTICIPANT &&
                        error.error !== DiscoveryError_1.default.NO_ENTRY_FOR_SELECTED_BACKENDS) {
                        log.error(`Discovery attempt for domains ${domains}, interface ${interfaceName}, gbids ${gbids} \
                            failed due to DiscoveryError ${error.error.name} : ${error.detailMessage}. \
                            Attempting no retry`);
                        break;
                    }
                    else {
                        log.info(`Discovery attempt for domains ${domains}, interface ${interfaceName}, gbids ${gbids} \
                            failed due to DiscoveryError ${error.error.name} : ${error.detailMessage}. \
                            Attempting retry in ${discoveryQos.discoveryRetryDelayMs} ms`);
                    }
                }
                else if (error.message) {
                    log.info(`Discovery attempt for domains ${domains}, interface ${interfaceName} failed due to DiscoveryError: ${error.name}. Attempting retry in ${discoveryQos.discoveryRetryDelayMs} ms`);
                    errorMsg = `${error.name} : ${error.message}`;
                }
            }
        } while (arbitrationDeadline - (Date.now() + discoveryRetryDelayMs) > 0);
        if (incompatibleVersionsFound.length > 0) {
            let message;
            if (gbids && gbids.length > 0) {
                message = `no compatible provider found within discovery timeout for domains "${JSON.stringify(domains)}", interface "${interfaceName}", gbids "${JSON.stringify(gbids)}" with discoveryQos "${JSON.stringify(discoveryQos)}"`;
            }
            else {
                message = `no compatible provider found within discovery timeout for domains "${JSON.stringify(domains)}", interface "${interfaceName}" with discoveryQos "${JSON.stringify(discoveryQos)}"`;
            }
            return Promise.reject(new NoCompatibleProviderFoundException_1.default({
                detailMessage: message,
                discoveredVersions: incompatibleVersionsFound,
                interfaceName
            }));
        }
        else {
            let message;
            if (gbids && gbids.length > 0) {
                message = `no provider found within discovery timeout for domains "${JSON.stringify(domains)}", interface "${interfaceName}", gbids "${JSON.stringify(gbids)}" with discoveryQos "${JSON.stringify(discoveryQos)}"${errorMsg !== undefined ? `. Error: ${errorMsg}` : ""}`;
            }
            else {
                message = `no provider found within discovery timeout for domains "${JSON.stringify(domains)}", interface "${interfaceName}" with discoveryQos "${JSON.stringify(discoveryQos)}"${errorMsg !== undefined ? `. Error: ${errorMsg}` : ""}`;
            }
            return Promise.reject(new DiscoveryException_1.default({
                detailMessage: message
            }));
        }
    }
    async discoverStaticCapabilities(domains, interfaceName, discoveryQos, proxyVersion) {
        try {
            const arbitratedCaps = [];
            if (this.staticCapabilities === undefined) {
                return Promise.reject(new Error("Exception while arbitrating: static capabilities are missing"));
            }
            else {
                for (let i = 0; i < this.staticCapabilities.length; ++i) {
                    const capability = this.staticCapabilities[i];
                    if (domains.indexOf(capability.domain) !== -1 &&
                        interfaceName === capability.interfaceName &&
                        capability.providerVersion.majorVersion === proxyVersion.majorVersion &&
                        capability.providerVersion.minorVersion >= proxyVersion.minorVersion &&
                        Arbitrator.checkSupportsOnChangeSubscriptions(capability, discoveryQos.providerMustSupportOnChange)) {
                        arbitratedCaps.push(capability);
                    }
                }
                return discoveryQos.arbitrationStrategy(arbitratedCaps);
            }
        }
        catch (e) {
            return Promise.reject(new Error(`Exception while arbitrating: ${e}`));
        }
    }
    /**
     * Shutdown the Arbitrator
     */
    shutdown() {
        for (const id in this.pendingArbitrations) {
            if (Object.prototype.hasOwnProperty.call(this.pendingArbitrations, id)) {
                const pendingArbitration = this.pendingArbitrations[id];
                pendingArbitration.reject(new Error("Arbitration is already shut down"));
            }
        }
        this.pendingArbitrations = {};
        this.started = false;
    }
}
module.exports = Arbitrator;
